#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Iterates over a bunch of .jpg or .cr2 files and matches 
DateTimeOriginal from Exif tag to DateTime in a csv log 
of a GeigerMuellerCounter. """

from datetime import datetime
import os
import csv
import argparse
import pyexiv2

# SIFACTOR for GQ Geiger counters

# 300  series: 0.0065   µSv/h / CPM
# 320  series: 0.0065   µSv/h / CPM
# 500  series: 0.0065   µSv/h / CPM
# 500+ series: 0.0065   µSv/h / CPM for the first tube
# 600  series: 0.0065   µSv/h / CPM
# 600+ series: 0.002637 µSv/h / CPM

# Configure argument parser for cli options
parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter, 
                                 description='''A tool that writes 
    radiation levels (and optionally geocoordinates) to image files 
    and extracts the infos from external sources.''')
parser.add_argument('-si', '--sifactor', type=float, default=0.0065, 
                    help='Factor to multiply recorded CPM with.')
parser.add_argument('csv', metavar='CSV', type=str, 
                    help='Geiger counter history file in CSV format.')
parser.add_argument('photos', metavar='Photo', type=str, nargs='+', 
                    help='One or multiple photo image files to process.')
parser.add_argument('-o', '--outdir', type=str, default='.', 
                    help='Directory to output processed photos')

args = parser.parse_args()

for photo in args.photos:
    # Load Exif data from image
    metadata = pyexiv2.ImageMetadata(photo)
    metadata.read()
    tag = metadata['Exif.Photo.DateTimeOriginal']
    # tag.value creates datetime object in pictime
    pictime = tag.value
    # Get image file name out of path
    photo_basename = os.path.basename(photo)
    
    # Import GeigerCounter log
    with open(args.csv, "r") as f:
        csvreader = csv.reader(filter(lambda row: row[0] != '#', f), 
                               delimiter=',', skipinitialspace=True)

        print('Processing file:', photo_basename, end='\r')
        for _, csvrawtime, csvrawcpm, _ in csvreader:
            csvtime = datetime.fromisoformat(csvrawtime)
            # Process image if its timestamp is found in CSV log
            if csvtime == pictime:
                rad = round(float(csvrawcpm) * args.sifactor, 2)

                # convert str to exif compatible string
                new_comment = 'Radiation ☢ ' + str(rad) + ' µS/h'
#                user_comment = piexif.helper.UserComment.dump(new_comment, encoding="unicode")
#                exif_dict["Exif"][piexif.ExifIFD.UserComment] = user_comment
                
                # compile and write tags
#                exif_bytes = piexif.dump(exif_dict)

                # be os aware and use the correct directory delimiter
                outfile = os.path.join(args.outdir, photo_basename)
                # write target file
                # print output
                print('{:<30} {:<20} {:<22}'.format(photo_basename, str(pictime), new_comment))
                break
        else:
            print('{:<30} {:<20} {:<22}'.format(photo_basename, str(pictime), 'NOT FOUND!'))
    # close CSV file
    f.close()
